#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" gradient_dict.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal


class Gradient_Dict(dict):
    """ A dictionary designed for efficient retrieval in large structures.
        Uses a gradient index allowing faster searching for keys. """

    pass
