#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" ## Implore

    >An even more desperate way to make requests for Humans.

    *Tell your computer to "Do What I Tell You!"*
    "Useful requests, automation, texting, and utilities for macOS"
    https://www.github.com/skeptycal/implore

    Part of the `AutoSys` package - utilities for macOS apps
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """

# __version__: str = "0.8.0"
# __title__: str = "autosys"
# __author__: str = "Michael Treanor"
# __author_email__: str = "skeptycal@gmail.com"
# __copyright__ = "Copyright (c) 2019 Michael Treanor"
# __license__: str = "MIT"
# __python_requires__: str = ">=3.8"

# __all__ = [
#     '__version__',
#     '__title__',
#     '__author__',
#     '__author_email__',
#     '__copyright__'
#     '__license__',
#     '__python_requires__'
# ]

"""
# *#####################################################################* #
### implore

>implore uses *Requests* and *Beautiful Soup* to parse html and send text messages and alerts based on preset conditions.

#### Features include:

    * Send text messages based on html results
    * NLP analysis of web pages or sets of pages
    * Test API functionality and build an interface
    * Create a command line interface
    * Research topics based on logic, common words, etc
    * Perform spoof analysis and fact checking
    * Convert websites to 3d object visuals
    * Collect website html code
    * Collect metadata and tag info
    * Watch for cookies and other local permissions issues
    * Create link and image catalogs
    * Interact with underlying database
    * Block specific activities of certain sites
    * Create a site map
    * Change CSS Design
    * Parse html and language results
    * Timer Intervals
    * Calendar Requests
    * Direct Request from user
    * Text File Scripts

Implore requires Python 3.6+ and works best with lxml(fast html) and ujson (fast json) installed. Text messaging requires macOS.

# *#####################################################################* #
### Beautiful Soup
_Elixir and Tonic_
"The Screen-Scraper's Friend"
[Beautiful Soup Documentation](http://www.crummy.com/software/BeautifulSoup/)
Copyright (c) 2004-2019 Leonard Richardson
MIT License

>Beautiful Soup uses a pluggable XML or HTML parser to parse a
>(possibly invalid) document into a tree representation. Beautiful Soup
>provides methods and Pythonic idioms that make it easy to navigate,
>search, and modify the parse tree.

For more than you ever wanted to know about Beautiful Soup, see the
documentation:
[Beautiful Soup Documentation](http://www.crummy.com/software/BeautifulSoup/bs4/doc/)

"""
# *#####################################################################* #
"""
### Requests: HTTP for Humans™

_Requests is an elegant and simple HTTP library for Python, built for human beings._

[Requests Documentation](https://requests.readthedocs.io/en/master/)

>Requests allows you to send HTTP/1.1 requests extremely easily. There’s no need to manually add query strings to your URLs, or to form-encode your POST data. Keep-alive and HTTP connection pooling are 100% automatic, thanks to urllib3.

### Beloved Features
>Requests is ready for today’s web.

- Keep-Alive & Connection Pooling
- International Domains and URLs
- Sessions with Cookie Persistence
- Browser-style SSL Verification
- Automatic Content Decoding
- Basic/Digest Authentication
- Elegant Key/Value Cookies
- Automatic Decompression
- Unicode Response Bodies
- HTTP(S) Proxy Support
- Multipart File Uploads
- Streaming Downloads
- Connection Timeouts
- Chunked Requests
- .netrc Support
"""
